unit hiIOportEx;

interface

uses Kol,Share,Debug,Windows,DLLLoader;

var
  Nport: Word;
  Nbit: Byte;
  IODll: PStream;
  LoadDll : TDLLLoader;
  tmpByte: Byte;
  tmpWord: Word;
  tmpDWord: DWord;


  PortOut : procedure (Port : Word; Data : Byte); stdcall;
  PortWordOut : procedure (Port : Word; Data : Word); stdcall;
  PortDWordOut : procedure (Port : Word; Data : DWord); stdcall;
  PortIn : function (Port : Word) : Byte; stdcall;
  PortWordIn : function (Port : Word) : Word; stdcall;
  PortDWordIn : function (Port : Word) : DWord; stdcall;
  SetPortBit : procedure (Port : Word; Bit : Byte); stdcall;
  ClrPortBit : procedure (Port : Word; Bit : Byte); stdcall;
  NotPortBit : procedure (Port : Word; Bit : Byte); stdcall;
  GetPortBit : function (Port : Word; Bit : Byte) : WordBool; stdcall;
  RightPortShift : function (Port : Word; Val : WordBool) : WordBool; stdcall;
  LeftPortShift : function (Port : Word; Val : WordBool) : WordBool; stdcall;
  IsDriverInstalled : function : Boolean; stdcall;

type
  THIIOportEx = class(TDebug)
   private

   public
    _data_Port:THI_Event;
    _data_Data:THI_Event;     
    _data_NumberBit:THI_Event;
    _data_Marker:THI_Event; 
    _event_onErr:THI_Event;
    _event_onRead:THI_Event;
    _event_onGetBit:THI_Event;
    _event_onMarker:THI_Event;
    _prop_Type:byte;
    _prop_Port:integer;

    constructor Create;
    destructor Destroy; override;
    procedure _work_doRead (var _Data:TData; Index:word);
    procedure _work_doWrite(var _Data:TData; Index:word);
    procedure _work_doPort (var _Data:TData; Index:word);
    procedure _work_doSetPortBit (var _Data:TData; Index:word);
    procedure _work_doClrPortBit (var _Data:TData; Index:word);
    procedure _work_doNotPortBit (var _Data:TData; Index:word);
    procedure _work_doGetPortBit (var _Data:TData; Index:word);
    procedure _work_doRightPortShift (var _Data:TData; Index:word);
    procedure _work_doLeftPortShift (var _Data:TData; Index:word);
    procedure Init;
  end;

implementation

{$I io.inc}

constructor THIIOportEx.Create;
begin
  inherited;
  InitAdd(Init);
end;

destructor THIIOportEx.Destroy;
begin
  LoadDll.Free;
  inherited;
end;

procedure THIIOportEx.Init;
var dt:TData;
begin
  LoadDll:=TDLLLoader.Create;
  IODll := NewMemoryStream;
  IODll.Write(ioData, ioSize);
  if LoadDll.Load(IODll) then
    begin
      PortOut            := LoadDll.FindExport('PortOut');
      PortWordOut        := LoadDll.FindExport('PortWordOut');
      PortDWordOut       := LoadDll.FindExport('PortDWordOut');
      PortIn             := LoadDll.FindExport('PortIn');
      PortWordIn         := LoadDll.FindExport('PortWordIn');
      PortDWordIn        := LoadDll.FindExport('PortDWordIn');
      SetPortBit         := LoadDll.FindExport('SetPortBit');
      ClrPortBit         := LoadDll.FindExport('ClrPortBit');
      NotPortBit         := LoadDll.FindExport('NotPortBit');
      GetPortBit         := LoadDll.FindExport('GetPortBit');
      RightPortShift     := LoadDll.FindExport('RightPortShift');
      LeftPortShift      := LoadDll.FindExport('LeftPortShift');
      IsDriverInstalled  := LoadDll.FindExport('IsDriverInstalled');
      if not IsDriverInstalled then begin
        _hi_OnEvent(_event_onErr, '   ');
        exit;
      end;
    end else begin
      _hi_OnEvent(_event_onErr, '   ');
      exit;
    end;
  IODll.Free;
  dt := _doData(_prop_Port);
  _work_doPort(dt,0);
end;

procedure THIIOportEx._work_doPort;
begin
  Nport := Word(ReadInteger(_Data,_data_Port,_prop_Port));
end;


procedure THIIOportEx._work_doRead;
begin
  case _prop_Type  of
  0: _hi_OnEvent(_event_onRead, PortIn(Nport));
  1: _hi_OnEvent(_event_onRead, PortWordIn(Nport));
  2: _hi_OnEvent(_event_onRead, PortDWordIn(Nport));
  end;
end;

procedure THIIOportEx._work_doWrite;
begin
  case _prop_Type  of
  0: begin
       tmpByte := Byte(ReadInteger(_Data,_data_Data));
       PortOut(Nport,tmpByte);
     end;
  1: begin
       tmpWord := Word(ReadInteger(_Data,_data_Data));
       PortWordOut(Nport,tmpWord);
     end;
  2: begin
       tmpDWord := DWord(ReadInteger(_Data,_data_Data));
       PortDWordOut(Nport,tmpDWord);
     end;
  end;
end;

procedure THIIOportEx._work_doSetPortBit;
begin
   Nbit := Byte(ReadInteger(_Data,_data_NumberBit));
   SetPortBit(Nport, Nbit);
end;

procedure THIIOportEx._work_doClrPortBit;
begin
  Nbit := Byte(ReadInteger(_Data,_data_NumberBit));
  ClrPortBit(Nport, Nbit);
end;

procedure THIIOportEx._work_doNotPortBit;
begin
  Nbit := Byte(ReadInteger(_Data,_data_NumberBit));
  NotPortBit(Nport, Nbit);
end;

procedure THIIOportEx._work_doGetPortBit;
begin
  Nbit := Byte(ReadInteger(_Data,_data_NumberBit));
  case GetPortBit(Nport, Nbit) of
   True:  _hi_OnEvent(_event_onGetBit, 1);
   False: _hi_OnEvent(_event_onGetBit, 0);
  end;   
end;

procedure THIIOportEx._work_doRightPortShift;
begin
  case RightPortShift(Nport, WordBool(ToIntegerEvent(_data_Marker))) of
   True:  _hi_OnEvent(_event_onMarker, 1);
   False: _hi_OnEvent(_event_onMarker, 0);
  end;  
end;

procedure THIIOportEx._work_doLeftPortShift;
begin
  case LeftPortShift(Nport, WordBool(ToIntegerEvent(_data_Marker))) of
   True:  _hi_OnEvent(_event_onMarker, 1);
   False: _hi_OnEvent(_event_onMarker, 0);
  end; 
end;

end.
